@@ -16,7 +16,7 @@
 
 #include <IntPatch_ALineToWLine.hxx>
 
-#include <Adaptor3d_HSurface.hxx>
+#include <Adaptor3d_Surface.hxx>
 #include <ElSLib.hxx>
 #include <IntPatch_ALine.hxx>
 #include <IntPatch_Point.hxx>
@@ -67,8 +67,8 @@ static void AddVertexPoint(Handle(IntSurf_LineOn2S)& theLine,
 //           theSingularSurfaceID contains the ID of surface with
 //            special point (0 - none, 1 - theS1, 2 - theS2)
 //=======================================================================
-static IntPatch_SpecPntType IsPoleOrSeam(const Handle(Adaptor3d_HSurface)& theS1,
-                                         const Handle(Adaptor3d_HSurface)& theS2,
+static IntPatch_SpecPntType IsPoleOrSeam(const Handle(Adaptor3d_Surface)& theS1,
+                                         const Handle(Adaptor3d_Surface)& theS2,
                                          const IntSurf_PntOn2S& thePIsoRef,
                                          Handle(IntSurf_LineOn2S)& theLine,
                                          IntPatch_Point &theVertex,
@@ -138,8 +138,8 @@ static IntPatch_SpecPntType IsPoleOrSeam(const Handle(Adaptor3d_HSurface)& theS1
 //function : IntPatch_ALineToWLine
 //purpose  : 
 //=======================================================================
-IntPatch_ALineToWLine::IntPatch_ALineToWLine(const Handle(Adaptor3d_HSurface)& theS1,
-                                             const Handle(Adaptor3d_HSurface)& theS2,
+IntPatch_ALineToWLine::IntPatch_ALineToWLine(const Handle(Adaptor3d_Surface)& theS1,
+                                             const Handle(Adaptor3d_Surface)& theS2,
                                              const Standard_Integer theNbPoints) :
   myS1(theS1),
   myS2(theS2),
@@ -676,7 +676,11 @@ void IntPatch_ALineToWLine::MakeWLine(const Handle(IntPatch_ALine)& theALine,
             IntPatch_Point aLVtx = theALine->Vertex(i);
             aLVtx.SetValue(aVertP2S);
             aLVtx.SetTolerance(aVertToler);
-            aLVtx.SetParameter(aNewVertexParam);
+            Standard_Real aParam = aLVtx.ParameterOnLine();
+            if (Abs(aParam - theLPar) <= Precision::PConfusion()) //in the case of closed curve,
+              aLVtx.SetParameter(-1); //we don't know yet the number of points in the curve
+            else
+              aLVtx.SetParameter(aNewVertexParam);
             aSeqVertex(++aNewVertID) = aLVtx;
             hasVertexBeenChecked(i) = Standard_True;
             isFound = Standard_True;
@@ -698,64 +702,59 @@ void IntPatch_ALineToWLine::MakeWLine(const Handle(IntPatch_ALine)& theALine,
       continue;
     }
 
-    //-----------------------------------------------------------------
-    //--  Computation of transitions of the line on two surfaces    ---
-    //-----------------------------------------------------------------
-    IntSurf_TypeTrans trans1,trans2;
-    {
-      Standard_Integer indice1;
-      Standard_Real dotcross;
-      gp_Pnt aPP0, aPP1;
-      //
-      trans1=IntSurf_Undecided;
-      trans2=IntSurf_Undecided;
-      //
-      indice1 = aLinOn2S->NbPoints()/3;
-      if(indice1<=2) {
-        indice1 = 2;
-      }
-      //
-      aPP1=aLinOn2S->Value(indice1).Value();
-      aPP0=aLinOn2S->Value(indice1-1).Value();
-      //
-      gp_Vec tgvalid(aPP0, aPP1);
-      gp_Vec aNQ1 = myQuad1.Normale(aPP0);
-      gp_Vec aNQ2 = myQuad2.Normale(aPP0);
-      //
-      dotcross = tgvalid.DotCross(aNQ2, aNQ1);
-      if (dotcross > myTolTransition) {
-        trans1 = IntSurf_Out;
-        trans2 = IntSurf_In;
-      }
-      else if(dotcross < -myTolTransition) { 
-        trans1 = IntSurf_In;
-        trans2 = IntSurf_Out;
-      } 
-    }
-
     //-----------------------------------------------------------------
     //--              W  L  i  n  e     c  r  e  a  t  i  o  n      ---
     //-----------------------------------------------------------------
     Handle(IntPatch_WLine) aWLine;
     //
-    if(theALine->TransitionOnS1() == IntSurf_Touch)  { 
-                                  aWLine = new IntPatch_WLine(aLinOn2S,
+    if(theALine->TransitionOnS1() == IntSurf_Touch)
+    {
+      aWLine = new IntPatch_WLine(aLinOn2S,
                                   theALine->IsTangent(),
                                   theALine->SituationS1(),
                                   theALine->SituationS2());
+      aWLine->SetCreatingWayInfo(IntPatch_WLine::IntPatch_WLImpImp);
     }
-    else if(theALine->TransitionOnS1() == IntSurf_Undecided)  { 
+    else if(theALine->TransitionOnS1() == IntSurf_Undecided)
+    {
       aWLine = new IntPatch_WLine(aLinOn2S, theALine->IsTangent());
+      aWLine->SetCreatingWayInfo(IntPatch_WLine::IntPatch_WLImpImp);
     }
-    else { 
+    else
+    {
+      //Computation of transitions of the line on two surfaces    ---
+      const Standard_Integer indice1 = Max(aLinOn2S->NbPoints() / 3, 2);
+      const gp_Pnt &aPP0 = aLinOn2S->Value(indice1 - 1).Value(), 
+                   &aPP1 = aLinOn2S->Value(indice1).Value();
+      const gp_Vec tgvalid(aPP0, aPP1);
+      const gp_Vec aNQ1(myQuad1.Normale(aPP0)), aNQ2(myQuad2.Normale(aPP0));
+
+      const Standard_Real dotcross = tgvalid.DotCross(aNQ2, aNQ1);
+      
+      IntSurf_TypeTrans trans1 = IntSurf_Undecided,
+                        trans2 = IntSurf_Undecided;
+      
+      if (dotcross > myTolTransition)
+      {
+        trans1 = IntSurf_Out;
+        trans2 = IntSurf_In;
+      }
+      else if (dotcross < -myTolTransition)
+      {
+        trans1 = IntSurf_In;
+        trans2 = IntSurf_Out;
+      }
+
       aWLine = new IntPatch_WLine(aLinOn2S, theALine->IsTangent(),
-                                  trans1, // aline->TransitionOnS1(),
-                                  trans2);  //aline->TransitionOnS2());
+                                  trans1, trans2);
+      aWLine->SetCreatingWayInfo(IntPatch_WLine::IntPatch_WLImpImp);
     }
 
     for(Standard_Integer i = aSeqVertex.Lower(); i <= aNewVertID; i++)
     {
-      const IntPatch_Point& aVtx = aSeqVertex(i);
+      IntPatch_Point aVtx = aSeqVertex(i);
+      if (aVtx.ParameterOnLine() == -1) //in the case of closed curve,
+        aVtx.SetParameter (aWLine->NbPnts()); //we set the last parameter
       aWLine->AddVertex(aVtx);
     }
 
@@ -763,7 +762,7 @@ void IntPatch_ALineToWLine::MakeWLine(const Handle(IntPatch_ALine)& theALine,
 
     //the method ComputeVertexParameters can reduce the number of points in <aWLine>
     aWLine->ComputeVertexParameters(myTol3D);
-
+    
     if (aWLine->NbPnts() > 1)
     {
       aWLine->EnablePurging(Standard_False);
