@@ -44,22 +44,28 @@
 #include <StepBasic_SolidAngleMeasureWithUnit.hxx>
 #include <StepBasic_SolidAngleUnit.hxx>
 #include <StepBasic_UncertaintyMeasureWithUnit.hxx>
+#include <StepData_GlobalFactors.hxx>
 #include <STEPConstruct_UnitContext.hxx>
 #include <StepGeom_GeomRepContextAndGlobUnitAssCtxAndGlobUncertaintyAssCtx.hxx>
 #include <StepRepr_GlobalUncertaintyAssignedContext.hxx>
 #include <StepRepr_GlobalUnitAssignedContext.hxx>
 #include <TCollection_HAsciiString.hxx>
-#include <UnitsMethods.hxx>
 
 //=======================================================================
 //function : STEPConstruct_UnitContext
 //purpose  : 
 //=======================================================================
-STEPConstruct_UnitContext::STEPConstruct_UnitContext() : done(Standard_False)
+STEPConstruct_UnitContext::STEPConstruct_UnitContext()
+: done(Standard_False),
+  lengthFactor(0.0),
+  planeAngleFactor(0.0),
+  solidAngleFactor(0.0),
+  areaFactor(0.0),
+  volumeFactor(0.0)
 {
   lengthDone = planeAngleDone = solidAngleDone = hasUncertainty = 
     areaDone = volumeDone = Standard_False;
-  //pdn file r_47-sd.stp initalize field.
+  //pdn file r_47-sd.stp initialize field.
   theUncertainty = RealLast();
 }
 
@@ -79,24 +85,26 @@ void STEPConstruct_UnitContext::Init(const Standard_Real Tol3d)
   Handle(TCollection_HAsciiString) contextType =
    new TCollection_HAsciiString("3D Context with UNIT and UNCERTAINTY");
 
-  // Units : LengthUnit and PlaneAngleUnit (no SolidAngleUnit appliable)
+  // Units : LengthUnit and PlaneAngleUnit (no SolidAngleUnit applicable)
 
   Handle(StepBasic_NamedUnit) lengthUnit;
   Standard_CString uName = 0;
   Standard_Boolean hasPref = Standard_True;
   StepBasic_SiPrefix siPref = StepBasic_spMilli;
-  switch ( Interface_Static::IVal ( "write.step.unit" ) ) {
-  case  1 : uName = "INCH";             break;
-  default :
-  case  2 :                             break;
-  case  4 : uName = "FOOT";             break;
-  case  5 : uName = "MILE";             break;
-  case  6 : hasPref = Standard_False;   break;
-  case  7 : siPref = StepBasic_spKilo;  break;
-  case  8 : uName = "MIL";              break;
-  case  9 : siPref = StepBasic_spMicro; break;
-  case 10 : siPref = StepBasic_spCenti; break;
-  case 11 : uName = "MICROINCH";        break;
+  Standard_Real aScale = 1.;
+  switch (Interface_Static::IVal("write.step.unit"))
+  {
+    case  1: uName = "INCH"; aScale = 25.4; break;
+    default:
+    case  2: break;
+    case  4: uName = "FOOT"; aScale = 304.8; break;
+    case  5: uName = "MILE"; aScale = 1609344.0; break;
+    case  6: hasPref = Standard_False; aScale = 1000.0; break;
+    case  7: siPref = StepBasic_spKilo; aScale = 1000000.0; break;
+    case  8: uName = "MIL"; aScale = 0.0254; break;
+    case  9: siPref = StepBasic_spMicro; aScale = 0.001; break;
+    case 10: siPref = StepBasic_spCenti; aScale = 10.0; break;
+    case 11: uName = "MICROINCH"; aScale = 0.0000254; break;
   }
   
   Handle(StepBasic_SiUnitAndLengthUnit) siUnit =
@@ -106,7 +114,7 @@ void STEPConstruct_UnitContext::Init(const Standard_Real Tol3d)
   if ( uName ) { // for non-metric units, create conversion_based_unit
     Handle(StepBasic_MeasureValueMember) val = new StepBasic_MeasureValueMember;
     val->SetName("LENGTH_UNIT");
-    val->SetReal ( UnitsMethods::GetLengthFactorValue ( Interface_Static::IVal ( "write.step.unit" ) ) );
+    val->SetReal (aScale);
 
     Handle(StepBasic_LengthMeasureWithUnit) measure = new StepBasic_LengthMeasureWithUnit;
     StepBasic_Unit Unit;
@@ -158,7 +166,7 @@ void STEPConstruct_UnitContext::Init(const Standard_Real Tol3d)
   
   Handle(StepBasic_MeasureValueMember) mvs = new StepBasic_MeasureValueMember;
   mvs->SetName("LENGTH_MEASURE");
-  mvs->SetReal ( Tol3d / UnitsMethods::LengthFactor() );
+  mvs->SetReal ( Tol3d / StepData_GlobalFactors::Intance().LengthFactor() );
   StepBasic_Unit Unit;
   Unit.SetValue ( lengthUnit );
   theTol3d->Init(mvs, Unit, TolName, TolDesc);
@@ -235,7 +243,7 @@ Standard_Boolean STEPConstruct_UnitContext::SiUnitNameFactor(const Handle(StepBa
   case StepBasic_sunSteradian: 
     return Standard_True;
   default:
-//	cout << "Unknown SiUnitName : " << aSiUnit->Name() << endl;
+//	std::cout << "Unknown SiUnitName : " << aSiUnit->Name() << std::endl;
     return Standard_False;
   }
 }
@@ -260,7 +268,7 @@ Standard_Integer STEPConstruct_UnitContext::ComputeFactors(const Handle(StepRepr
   
   if (aContext.IsNull()) {
 #ifdef OCCT_DEBUG
-    cout<<" -- STEPConstruct_UnitContext:ComputeFactor, Context undefined -> default"<<endl;
+    std::cout<<" -- STEPConstruct_UnitContext:ComputeFactor, Context undefined -> default"<<std::endl;
 #endif
     return 1;
   }
@@ -274,7 +282,7 @@ Standard_Integer STEPConstruct_UnitContext::ComputeFactors(const Handle(StepRepr
     status = ComputeFactors(theNamedUnit);
 #ifdef OCCT_DEBUG
     if(status == -1)
-      cout << " -- STEPConstruct_UnitContext:ComputeFactor: Unit item no." << i << " is not recognized" << endl;
+      std::cout << " -- STEPConstruct_UnitContext:ComputeFactor: Unit item no." << i << " is not recognized" << std::endl;
 #endif    
   }
   return status;
@@ -327,11 +335,11 @@ Standard_Integer STEPConstruct_UnitContext::ComputeFactors(const Handle(StepBasi
 	}
 	// Treat the SiUnitName
 	if (!SiUnitNameFactor(theSIU,theSIUNF)) status = 11; // et continue
-	//cout << "The SiUnitNameFactor is :";
-	//cout << theSIUNF << endl;
+	//std::cout << "The SiUnitNameFactor is :";
+	//std::cout << theSIUNF << std::endl;
       }
       else {
-	//      cout << "Recursive algo required - Aborted" << endl;
+	//      std::cout << "Recursive algo required - Aborted" << std::endl;
 	return 3;
       }
       Standard_Real theMVAL = theMWU->ValueComponent();
@@ -344,7 +352,7 @@ Standard_Integer STEPConstruct_UnitContext::ComputeFactors(const Handle(StepBasi
     else {
       status = 14;
 #ifdef OCCT_DEBUG
-      cout << "Error in the file : parameter double defined" << endl;
+      std::cout << "Error in the file : parameter double defined" << std::endl;
 #endif
     }
   }    
@@ -369,7 +377,7 @@ Standard_Integer STEPConstruct_UnitContext::ComputeFactors(const Handle(StepBasi
     else {
       status = 14;
 #ifdef OCCT_DEBUG
-      cout << "Error in the file : parameter double defined" << endl;
+      std::cout << "Error in the file : parameter double defined" << std::endl;
 #endif
     }
   }
@@ -377,24 +385,25 @@ Standard_Integer STEPConstruct_UnitContext::ComputeFactors(const Handle(StepBasi
   // Defining a type of unit
   if(!parameterDone) {
 #ifdef OCCT_DEBUG
-    cout << "Unit Type not implemented" << endl;
+    std::cout << "Unit Type not implemented" << std::endl;
 #endif 
     return 0;
   }
   
+  const Standard_Real aCascadeUnit = StepData_GlobalFactors::Intance().CascadeUnit();
   if (aUnit->IsKind(STANDARD_TYPE(StepBasic_ConversionBasedUnitAndLengthUnit))||
       aUnit->IsKind(STANDARD_TYPE(StepBasic_SiUnitAndLengthUnit))) {
 #ifdef METER
     lengthFactor = parameter;
 #else
-    lengthFactor = parameter * 1000. / UnitsMethods::GetCasCadeLengthUnit();
+    lengthFactor = parameter * 1000. / aCascadeUnit;
 #endif    
     if(!lengthDone) 
       lengthDone = Standard_True;
     else {
       status = 14;
 #ifdef OCCT_DEBUG
-      cout << "Error in the file : LengthFactor double defined" << endl;
+      std::cout << "Error in the file : LengthFactor double defined" << std::endl;
 #endif    
     }
   }  // end of LengthUnit
@@ -414,7 +423,7 @@ Standard_Integer STEPConstruct_UnitContext::ComputeFactors(const Handle(StepBasi
 #ifdef METER   
     af = parameter;
 #else
-    af = parameter * 1000. / UnitsMethods::GetCasCadeLengthUnit();
+    af = parameter * 1000. / aCascadeUnit;
 #endif    
     areaDone = Standard_True;
     areaFactor = pow(af,2);
@@ -425,7 +434,7 @@ Standard_Integer STEPConstruct_UnitContext::ComputeFactors(const Handle(StepBasi
 #ifdef METER   
     af = parameter;
 #else
-    af = parameter * 1000. / UnitsMethods::GetCasCadeLengthUnit();
+    af = parameter * 1000. / aCascadeUnit;
 #endif
     volumeDone = Standard_True;
     volumeFactor = pow(af,3);
@@ -455,7 +464,7 @@ Standard_Integer STEPConstruct_UnitContext::ComputeTolerance
     Handle(StepBasic_UncertaintyMeasureWithUnit) aUMWU = aContext->UncertaintyValue(un);
     if (aUMWU.IsNull()) {
 #ifdef OCCT_DEBUG
-      cout<<"BAD Uncertainty Measure with Units, n0."<<un<<endl;
+      std::cout<<"BAD Uncertainty Measure with Units, n0."<<un<<std::endl;
 #endif
       continue;
     }
@@ -485,7 +494,7 @@ Standard_Integer STEPConstruct_UnitContext::ComputeTolerance
   }
 
 #ifdef OCCT_DEBUG
-  if (hasUncertainty) cout << "UNCERTAINTY read as " << theUncertainty << endl;
+  if (hasUncertainty) std::cout << "UNCERTAINTY read as " << theUncertainty << std::endl;
 #endif
   return status;
 }
